var prefs = {
  //for messenger toolbar
  icon_mode: 1,
  placement: [],
  apply_multi: false,
  //for message window toolbar
  icon_mode_mw: 1,
  placement_mw: []
};

//normal: messenger, messageDisplay: messageWindow
var currentWindowType = "normal";

browser.windows.onCreated.addListener(async (win) => {
  if (win.type == "normal") {
    await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs.placement);
    await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs.icon_mode);
  } else if (win.type == "messageDisplay") {
    await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs.placement_mw);
    await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs.icon_mode_mw);
  }
});

browser.storage.onChanged.addListener(async (changes, area) => {
  let msgrChanged = false;
  let msgwinChanged = false;

  for (let item in changes) {
    prefs[item] = changes[item].newValue;
    msgrChanged = msgrChanged || (item.indexOf("_mw") == -1);
    msgwinChanged = msgwinChanged || (item.indexOf("_mw") != -1);
  }

  if (msgrChanged) {
    changeHeaderForWindows("normal");
  }

  if (msgwinChanged) {
    changeHeaderForWindows("messageDisplay");
  }
});

browser.runtime.onMessage.addListener(async (message) => {
  switch (message.message) {
    case "GET_BUTTONS":
      let buttons = [];
      let wins = await browser.windows.getAll();
      for (let i = 0; i < wins.length; i++) {
        let win = wins[i];
        if (win.type == currentWindowType) {
          buttons = await browser.msgHdrCustomizeApi.getButtons(win.id);
          break;
        }
      }
      browser.runtime.sendMessage({
        message: "SEND_BUTTONS",
        buttons: buttons,
        type: currentWindowType
      });
      break;
    case "CLOSE_REQ":
      await browser.windows.remove(message.windowId);
      break;  
    default:
      break;
  }
});

async function changeHeaderForWindows(type) {
  let wins = await browser.windows.getAll();
  for (let i = 0; i < wins.length; i++) {
    let win = wins[i];
    if (win.type == type) {
      let suffix = (win.type == "messageDisplay") ? "_mw" : "";
      await browser.msgHdrCustomizeApi.reorderButtons(win.id, prefs["placement" + suffix]);
      await browser.msgHdrCustomizeApi.setIconMode(win.id, prefs["icon_mode" + suffix]);
    }
  }
}

async function init() {
  for (let key in prefs) {
    let result = await browser.storage.local.get(key);
    let value = result[key];
    if (value !== undefined) {
      prefs[key] = value;
    }
  }

  await browser.msgHdrCustomizeApi.init();

  browser.msgHdrCustomizeApi.onMenuClick.addListener(async (id) => {
    if (id.indexOf("MSG") != -1) {
      currentWindowType = "normal";
    } else if (id.indexOf("MW") != -1) {
      currentWindowType = "messageDisplay";
    }
    await browser.windows.create({
      height: 450,
      width: 390,
      url: "html/options_window.html",
      type: "popup"
    });
  });

  browser.msgHdrCustomizeApi.onLoadMultiselect.addListener(async (id) => {
    if (prefs.apply_multi) {
      await browser.msgHdrCustomizeApi.reorderButtonsMulti(prefs.placement);
      await browser.msgHdrCustomizeApi.setIconModeMulti(prefs.icon_mode);
    }
  });

  //for just after install
  changeHeaderForWindows("normal");
  changeHeaderForWindows("messageDisplay");
}

init();
