var prefSuffix = "";

async function init() {
  translate();
  await browser.runtime.sendMessage({
    message: "GET_BUTTONS"
  });

  let select = document.mhc_form.icon_mode;
  select.addEventListener("change", async () => {
    let mode = document.mhc_form.icon_mode.value;
    let obj = {};
    obj["icon_mode" + prefSuffix] = parseInt(mode);
    await browser.storage.local.set(obj);
  });

  let result = await browser.storage.local.get("apply_multi");
  let check = document.getElementById("apply_multi");
  check.checked = result.apply_multi ? true : false;
  check.addEventListener("change", async () => {
    let checked = document.getElementById("apply_multi").checked;
    await browser.storage.local.set({
      apply_multi: checked
    });
  });

  let buttonList = document.getElementById("placement");
  buttonList.addEventListener("change", async () => {
    updateShowHideButton();
    enableButtons();
  });

  document.getElementById("buttonClose").addEventListener("click", async () => {
    let windowId = (await browser.windows.getCurrent()).id;
    await browser.runtime.sendMessage({
      message: "CLOSE_REQ",
      windowId: windowId
    });
  });
}

browser.runtime.onMessage.addListener(async (message) => {
  switch (message.message) {
    case "SEND_BUTTONS":
      prefSuffix = (message.type == "normal") ? "" : "_mw";
      setIconMode();
      fillButtonList(message.buttons);
      if (prefSuffix == "") {
        document.getElementById("multiselArea").classList.remove("box-collapse");
        document.getElementById("multiselArea").classList.add("box-visible");
      }
      break;
    default:
      break;
  }
});

function setIconToOption(id, url) {
  let css = `
  #${id}:before{
    content: "";
    display: inline-block;
    width: 16px;
    height: 16px;
    background-image: ${url};
    background-repeat: no-repeat;
    background-size: contain;
    vertical-align: middle;
    padding-right: 5px;
  }`;

  document.styleSheets[0].insertRule(css);
}

async function setIconMode() {
  let result = await browser.storage.local.get("icon_mode" + prefSuffix);
  let select = document.mhc_form.icon_mode;
  select.value = result["icon_mode" + prefSuffix] === undefined ? 1 : result["icon_mode" + prefSuffix];
}

function fillButtonList(buttons) {
  let box = document.getElementById("placement");

  for (let i = box.childNodes.length - 1; i >= 0; i--) {
    box.removeChild(box.childNodes[i]);
  }

  let hiddenMsg = browser.i18n.getMessage("mhcHidden");
  for (let i = 0; i < buttons.length; i++) {
    let button = buttons[i];
    let option = document.createElement("option");
    if (button.id.indexOf("mhcSPC") == 0) {
      option.textContent = browser.i18n.getMessage("mhcSpace");
      option.classList.add("spacer");
    } else if (button.id.indexOf("mhcSEP") == 0) {
      option.textContent = browser.i18n.getMessage("mhcSeparator");
      option.classList.add("spacer");
    } else {
      option.textContent = button.collapsed ? button.label + " " + hiddenMsg : button.label;
      setIconToOption(button.id, button.iconURL);
    }
    option.id = button.id;
    option.setAttribute("buttonLabel", button.label);
    option.setAttribute("buttonCollapsed", button.collapsed);

    if (button.collapsed) {
      option.classList.add("option-hidden");
    }

    box.appendChild(option);
  }

  document.getElementById("buttonUp").addEventListener("click", () => {
    let moved = moveOptsInSelection(
      document.getElementById("placement"),
      true
    );
    if (moved) updateButtonOrder();
  });

  document.getElementById("buttonDown").addEventListener("click", () => {
    let moved = moveOptsInSelection(
      document.getElementById("placement"),
      false
    );
    if (moved) updateButtonOrder();
  });

  document.getElementById("buttonHide").addEventListener("click", () => {
    let changed = hideSelectedButtons(
      document.getElementById("placement")
    );
    if (changed) {
      updateShowHideButton();
      updateButtonOrder();
    }
  });

  document.getElementById("buttonSpc").addEventListener("click", async () => {
    let changed = addSpacer(
      document.getElementById("placement"),
      "mhcSPC"
    );
    if (changed) updateButtonOrder();
  });

  document.getElementById("buttonSep").addEventListener("click", async () => {
    let changed = addSpacer(
      document.getElementById("placement"),
      "mhcSEP"
    );
    if (changed) updateButtonOrder();
  });

  document.getElementById("buttonReset").addEventListener("click", async () => {
    await resetButtonOrder();
  });

  enableButtons();
}

function enableButtons() {
  let select = document.getElementById("placement");
  let opts = select.selectedOptions;

  if (opts.length > 0) {
    document.getElementById("buttonHide").disabled = false;
    document.getElementById("buttonSpc").disabled = (opts.length > 1);
    document.getElementById("buttonSep").disabled = (opts.length > 1);
    let firstIndex = getIndexOfOption(select, opts[0]);
    document.getElementById("buttonUp").disabled = (firstIndex == 0);
    let lastIndex = getIndexOfOption(select, opts[opts.length - 1]);
    document.getElementById("buttonDown").disabled = (lastIndex == select.options.length - 1);
  } else {
    document.getElementById("buttonUp").disabled = true;
    document.getElementById("buttonDown").disabled = true;
    document.getElementById("buttonHide").disabled = true;
    document.getElementById("buttonSpc").disabled = true;
    document.getElementById("buttonSep").disabled = true;
  }
}

function moveOptsInSelection(list, up) {
  if (list.selectedIndex == -1) return false;
  let orgOpts = list.selectedOptions;

  //copy to array. selectedOptions will change by the appendChild() and insertBefore()
  let opts = [];
  for (let i = 0; i < orgOpts.length; i++) {
    opts.push(orgOpts[i]);
  }

  let firstIndex = getIndexOfOption(list, opts[0]);
  let lastIndex = getIndexOfOption(list, opts[opts.length - 1]);
  if (up && firstIndex == 0) return false;
  if (!up && lastIndex == list.childNodes.length - 1) return false;

  if (up) {
    for (let i = 0; i < opts.length; i++) {
      let opt = opts[i];
      swapOptInSelection(list, up, opt);
    }
  } else {
    for (var i = opts.length - 1; i >= 0; i--) {
      let opt = opts[i];
      swapOptInSelection(list, up, opt);
    }
  }

  enableButtons();

  return true;
}

function swapOptInSelection(list, up, opt) {
  let refIndex = up ? getIndexOfOption(list, opt) - 1 : getIndexOfOption(list, opt) + 2;
  let refChild = list.childNodes[refIndex];
  if (refIndex >= list.childNodes.length) {
    list.appendChild(opt);
  } else {
    list.insertBefore(opt, refChild);
  }
}

function getIndexOfOption(list, opt) {
  let index = -1;
  for (let i = 0; i < list.length; i++) {
    if (list[i] === opt) {
      index = i;
      break;
    }
  }

  return index;
}

async function updateButtonOrder() {
  let box = document.getElementById("placement");
  let newOrder = [];
  for (let i = 0; i < box.options.length; i++) {
    let opt = box.options[i];
    newOrder.push({
      id: opt.id,
      label: opt.getAttribute("buttonLabel"),
      collapsed: (opt.getAttribute("buttonCollapsed") == "true")
    });
  }

  let obj = {};
  obj["placement" + prefSuffix] = newOrder;
  await browser.storage.local.set(obj);
}

function hideSelectedButtons(list) {
  if (list.selectedIndex == -1) return false;

  let hiddenMsg = browser.i18n.getMessage("mhcHidden");
  let opts = list.selectedOptions;

  let hide = !(opts[0].getAttribute("buttonCollapsed") == "true"); //adopt top button for hidden mode
  for (let i = opts.length - 1; i >= 0; i--) {
    let opt = opts[i];
    if (hide) {
      if (opt.id.indexOf("mhcSPC") == 0 || opt.id.indexOf("mhcSEP") == 0) {
        list.removeChild(opt);
      } else {
        opt.textContent = opt.getAttribute("buttonLabel") + " " + hiddenMsg;
        opt.setAttribute("buttonCollapsed", true);
        opt.classList.add("option-hidden");
      }
    } else {
      opt.textContent = opt.getAttribute("buttonLabel");
      opt.setAttribute("buttonCollapsed", false);
      opt.classList.remove("option-hidden");
    }
  }

  return true;
}

async function resetButtonOrder() {
  let obj = {};
  obj["placement" + prefSuffix] = [];
  await browser.storage.local.set(obj);

  let msg = browser.i18n.getMessage("mhcRestartMsg");
  alert(msg);
}

function addSpacer(list, type) {
  if (list.selectedIndex == -1) return false;
  let refOpt = list.selectedOptions[0];
  let label = "";
  if (type == "mhcSPC") {
    label = browser.i18n.getMessage("mhcSpace");
  } else if (type == "mhcSEP") {
    label = browser.i18n.getMessage("mhcSeparator");
  }
  let opt = document.createElement("option");
  opt.textContent = label;
  opt.setAttribute("buttonCollapsed", false);
  let date = new Date();
  let id = date.getTime();
  opt.id = type + "-" + id;
  opt.setAttribute("buttonLabel", label);

  opt.classList.add("spacer");

  list.insertBefore(opt, refOpt);

  return true;
}

function updateShowHideButton() {
  let box = document.getElementById("placement");
  let opt = box.selectedOptions[0];
  let localeKey = "";
  if (opt) {
    let hide = !(opt.getAttribute("buttonCollapsed") == "true");
    localeKey = hide ? "mhcHide" : "mhcShow";
  } else { //hiding space and separator case. opt is removed.
    localeKey = "mhcHide";
  }

  document.getElementById("buttonHide").value = browser.i18n.getMessage(localeKey);
}

init();
